package com.elitesland.router.controller;

import com.elitesland.router.bean.GatewayRouteDefinition;
import com.elitesland.router.service.DynamicRouteService;
import com.elitesland.router.service.GatewayRoutesInNacosConfigService;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.config.PropertiesRouteDefinitionLocator;
import org.springframework.cloud.gateway.route.*;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.cloud.gateway.support.ServerWebExchangeUtils;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ServerWebExchange;
import org.springframework.web.server.ServerWebExchangeDecorator;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;

/*
 *@Author Horbin.Zhang
 *@Date 2021-02-10 19:17
 *@Description
 * http://localhost:9040/actuator/gateway/routes 通过acutator查询route配置信息
 */

@RestController
@RequestMapping("/route")
public class RouteController {
    @Autowired
    private GatewayRoutesInNacosConfigService gatewayRoutesInNacosConfigService;
    @Autowired
    PropertiesRouteDefinitionLocator PropertiesRouteDefinitionLocator;

    /**
     *增加路由
     */
    @PostMapping("/addOrUpdate")
    public Mono<GatewayRouteDefinition> addOrUpdate(@RequestBody GatewayRouteDefinition gwdefinition) {
        return Mono.just(gatewayRoutesInNacosConfigService.addOrUpdateRouteDefinitiToNacosServer(gwdefinition));
    }

    /**
     *删除路由
     */
    @DeleteMapping("/routes/{id}")
    public Mono<String> delete(@PathVariable String id) {
        return Mono.just(gatewayRoutesInNacosConfigService.deleteRouteDefinitiToNacosServer(id));
    }

    /**
     * 获取nacos里配置的动态路由配置信息
     */
    @GetMapping("/routes/all")
    public Flux<GatewayRouteDefinition> routes() {
        Gson g = new Gson();
        ArrayList<GatewayRouteDefinition> list = new ArrayList<>();
        JsonArray gatewayDefinitionsFromNacosServer = gatewayRoutesInNacosConfigService.getGatewayDefinitionsFromNacosServer();
        gatewayDefinitionsFromNacosServer.forEach(o ->{
            GatewayRouteDefinition gatewayRouteDefinition = g.fromJson(o, GatewayRouteDefinition.class);
            list.add(gatewayRouteDefinition);
        });
        return Flux.fromIterable(list);
    }

    /**
     * 获取本地所配置文件中所有的静态路由配置信息
     */
    @GetMapping("staticRoutes/all")
    public Flux<RouteDefinition> staticRoutes() {
        return PropertiesRouteDefinitionLocator.getRouteDefinitions();
    }
}